/*
 * nasd_readwrite.c
 *
 * IO operations for EDRFS.
 *
 * Author: Nat Lanza
 */
/*
 * NASD is a copyright (c) of Carnegie Mellon University, 1999.
 *             
 * Permission to reproduce, use and prepare derivative works of
 * NASD for use is granted provided the copyright and "No Warranty"
 * statements are included with all reproductions and derivative
 * works. NASD may also be redistributed provided that the
 * copyright and "No Warranty" statements are included in all
 * redistributions.
 *             
 * NO WARRANTY. NASD IS FURNISHED ON AN "AS IS" BASIS. CARNEGIE
 * MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
 * LIMITED TO, WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY,
 * EXCLUSIVITY OF RESULTS OR RESULTS OBTAINED FROM USE OF NASD.
 * CARNEGIE MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY
 * KIND WITH RESPECT TO FREEDOM FROM PATENT, TRADEMARK OR
 * COPYRIGHT INFRINGEMENT.
 */



#include <nasd/nasd_options.h>
#include <nasd/nasd_sys.h>
#include <nasd/nasd_types.h>
#include <nasd/nasd_mem.h>
#include <nasd/nasd_threadstuff.h>
#include <nasd/nasd_freelist.h>
#include <nasd/nasd_edrfs_client.h>

#include <nasd/linux/nasd_edrfs_client_linux.h>
#include <nasd/linux/nasd_edrfs_client_linux_mount.h>

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/locks.h>
#include <linux/unistd.h>
#include <linux/pagemap.h>

#include <asm/system.h>
#include <asm/uaccess.h>

#include <linux/fs.h>
#include <linux/dcache.h>

#define DEBUG_READ_BASIC_DETAIL       1
#define DEBUG_READ_BASIC_DETAIL_EXTRA 1
#define DEBUG_WRITE_BASIC_DETAIL      0
#define DEBUG_READ_RANGE_DETAIL       0
#define DEBUG_WRITE_RANGE_DETAIL      0

nasd_status_t nasd_edrfs_read_basic(struct inode  *inode,
				    void          *buf,
				    int            buffer_space,
				    nasd_len_t     in_len,
				    nasd_offset_t  offset,
				    nasd_len_t    *out_len) {
  nasd_edrfs_inode_info_t *inode_info;
  nasd_edrfs_sb_info_t    *sb_info;
  
  nasd_edrfs_credential_t  in_credential;
  nasd_error_string_t      err_str;
  nasd_rpc_status_t        op_status;
  nasd_cookie_t            cookie;
  nasd_status_t            rc;
  char                    *my_buf = NULL;
  
  nasd_security_param_t    sp;
  nasd_p_smpl_op_dr_args_t read_args;
  nasd_p_fastread_dr_res_t read_res;
  
  NASD_ASSERT(inode   != NULL);
  NASD_ASSERT(buf     != NULL);
  NASD_ASSERT(out_len != NULL);
  
  inode_info = NASD_EDRFS_INODE_INFO(inode);
  sb_info = NASD_EDRFS_SUPER_SBINFO(inode->i_sb);
  
#if DEBUG_READ_BASIC_DETAIL
  nasd_printf("nasd_edrfs_read_basic asked to read %Lu bytes at offset %ld from nasdid %" NASD_ID_FMT "\n",
	      in_len, offset, inode_info->ident.nasd_identifier);
#endif /* DEBUG_READ_BASIC_DETAIL */ 
  
  if (buffer_space == NASD_EDRFS_BUFFER_USER) {
    /* buf is userspace, so get a kernel buffer to read into */
    NASD_Malloc(my_buf, in_len, (char *));
    
    if (my_buf == NULL) {
      nasd_printf("NASD-EDRFS: read_basic couldn't get memory!\n");
      rc = NASD_NO_MEM;
      goto out_nomem;
    }
  } else {
    my_buf = buf;
  }
  
  /* set up RPC */
  
  /* bail if our drive handle is bad. */
  if (!(sb_info->handles.flags & NASD_BINDING_DRIVE_VALID)) {
    rc = NASD_BAD_HANDLE;
    goto out;
  }
  
  in_credential.uid = current->uid;
  in_credential.gid = current->gid;
  
  /* get a cookie */
  rc = nasd_edrfs_find_or_get_cookie(&sb_info->handles, &inode_info->ident,
				     &in_credential, &cookie);
  if (rc != NASD_SUCCESS) { goto out; }
  

  read_args.in_identifier      = inode_info->ident.nasd_identifier;
  read_args.in_partnum         = inode_info->ident.partnum;
  read_args.in_offset          = offset;
  read_args.in_len             = in_len;

  SETUP_SECURITY_PARAM(sp, cookie.capability);

  nasd_cl_p_read_simple_dr(sb_info->handles.drive_handle,
			   cookie.key, &sp, &cookie.capability,
			   &read_args,
			   my_buf, &read_res, &op_status);
  rc = read_res.nasd_status;
  *out_len = read_res.out_datalen;

#if DEBUG_READ_BASIC_DETAIL
  nasd_printf("  read_simple returned, read %d bytes\n", *out_len);
#endif /* DEBUG_READ_BASIC_DETAIL */

  if ((rc != NASD_SUCCESS) || (op_status != 0)) {
    nasd_printf("NASD-EDRFS: read_basic: nasd_status=0x%x (%s), op_status=0x%x (%s)\n",
		rc, nasd_error_string(rc), op_status,
		nasd_cl_error_string(sb_info->handles.drive_handle,
				     op_status, err_str));
    goto out;
  }

  /* sigh */
  if (buffer_space == NASD_EDRFS_BUFFER_USER) {
    copy_to_user(buf, my_buf, read_res.out_datalen);
  }

 out:
  if (buffer_space == NASD_EDRFS_BUFFER_USER) {
    NASD_Free(my_buf, in_len);
  }
 out_nomem:
  return rc; 
}     


nasd_status_t nasd_edrfs_write_basic(struct inode  *inode,
				     void          *buf,
				     int            buffer_space,
				     nasd_len_t     in_len,
				     nasd_offset_t  offset,
				     nasd_len_t    *out_len) {
  nasd_edrfs_inode_info_t *inode_info;
  nasd_edrfs_sb_info_t    *sb_info;
  
  nasd_edrfs_credential_t  in_credential;
  nasd_error_string_t      err_str;
  nasd_rpc_status_t        op_status;
  nasd_cookie_t            cookie;
  nasd_status_t            rc;
  char                    *my_buf = NULL;

  nasd_security_param_t	sp;
  nasd_p_smpl_op_dr_args_t write_args;
  nasd_p_fastwrite_dr_res_t write_res;

  NASD_ASSERT(inode   != NULL);
  NASD_ASSERT(buf     != NULL);
  NASD_ASSERT(out_len != NULL);

  inode_info = NASD_EDRFS_INODE_INFO(inode);
  sb_info = NASD_EDRFS_SUPER_SBINFO(inode->i_sb);

#if DEBUG_WRITE_BASIC_DETAIL
  nasd_printf("nasd_edrfs_write(0x%lx, 0x%lx, %Lu, %Lu, 0x%lx) [ino=%" NASD_ID_FMT " %Lu@%Lu]\n",
	      (unsigned long) inode, (unsigned long) buf, in_len, offset,
	      (unsigned long) out_len,
	      inode_info->ident.nasd_identifier, in_len, offset);
#endif /* DEBUG_WRITE_BASIC_DETAIL */ 

  if (buffer_space == NASD_EDRFS_BUFFER_USER) {
    NASD_Malloc(my_buf, in_len, (char *));

    if (my_buf == NULL) {
      nasd_printf("NASD-EDRFS: read_basic couldn't get memory!\n");
      rc = NASD_NO_MEM;
      goto out_nomem;
    }

    copy_from_user(my_buf, buf, in_len);

  } else {
    my_buf = buf;
  }
     

  /* set up RPC */

  /* bail if our drive handle is bad. */
  if (!(sb_info->handles.flags & NASD_BINDING_DRIVE_VALID)) {
    rc = NASD_BAD_HANDLE;
    goto out;
  }

  in_credential.uid = current->uid;
  in_credential.gid = current->gid;

  /* get a cookie */
  rc = nasd_edrfs_find_or_get_cookie(&sb_info->handles, &inode_info->ident,
				     &in_credential, &cookie);
  if (rc != NASD_SUCCESS) { goto out; }

  write_args.in_identifier      = inode_info->ident.nasd_identifier;
  write_args.in_partnum         = inode_info->ident.partnum;
  write_args.in_offset          = offset;
  write_args.in_len             = in_len;

  SETUP_SECURITY_PARAM(sp, cookie.capability);

  nasd_cl_p_write_simple_dr(sb_info->handles.drive_handle,
			    cookie.key, &sp, &cookie.capability,
			    &write_args,
			    my_buf, &write_res, &op_status);
  rc = write_res.nasd_status;
  *out_len = write_res.out_datalen;

  if ((rc != NASD_SUCCESS) || (op_status != 0)) {
    nasd_printf("NASD-EDRFS: write_basic: nasd_status=0x%x (%s), op_status=0x%x (%s)\n",
		rc, nasd_error_string(rc), op_status,
		nasd_cl_error_string(sb_info->handles.drive_handle,
				     op_status, err_str));
  }
  
 out:  
  if (buffer_space == NASD_EDRFS_BUFFER_USER) {
    NASD_Free(my_buf, in_len);
  }  
 out_nomem:
  return rc;
}


nasd_status_t nasd_edrfs_read_cached(struct inode  *inode,
				     void          *buf,
				     int            buffer_space,
				     nasd_len_t     in_len,
				     nasd_offset_t  offset,
				     nasd_len_t    *out_len) {
  return nasd_edrfs_read_basic(inode, buf, buffer_space, in_len, offset, out_len);
}


nasd_status_t nasd_edrfs_read_uncached(struct inode  *inode,
				       void          *buf,
				       int            buffer_space,
				       nasd_len_t     in_len,
				       nasd_offset_t  offset,
				       nasd_len_t    *out_len) {
  return nasd_edrfs_read_basic(inode, buf, buffer_space, in_len, offset, out_len);
}

nasd_status_t nasd_edrfs_write_uncached(struct inode  *inode,
					void          *buf,
					int            buffer_space,
					nasd_len_t     in_len,
					nasd_offset_t  offset,
					nasd_len_t    *out_len) {
  return nasd_edrfs_write_basic(inode, buf, buffer_space, in_len, offset, out_len);
}

nasd_status_t nasd_edrfs_write_cached(struct inode  *inode,
				      void          *buf,
				      int            buffer_space,
				      nasd_len_t     in_len,
				      nasd_offset_t  offset,
				      nasd_len_t    *out_len) {
  return nasd_edrfs_write_basic(inode, buf, buffer_space, in_len, offset, out_len);
}
